-- Copyright 2023 DJ WoodZ

-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:

-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.

local AntiAFK = {
  lastTime = 0,
  adjustTime = 0,
  lastInputTime = 0,
  lastInputType = "",
  lastGamepadState = "",
  enabled = false,
  afkTimeout = 1000 * 60 * 15,
  afkWarningTime = 1000 * 60,
  textSize = 0.025,
  overlay = Overlay.new(HUD.MENU_BACKGROUND_PATH, 0, 0, 0, 0),
  debugMode = false,
  requestLeave = false,
}

function AntiAFK:getTime()
  -- same format as g_time, but more accurate on slow systems
  local time = g_networkTime
  local adjustedTime = time + self.adjustTime

  if adjustedTime < self.lastTime then
    -- update adjustment and advance by 1 second
    self.adjustTime = self.adjustTime + (self.lastTime - adjustedTime) + 1000
    adjustedTime = time + self.adjustTime
  end

  self.lastTime = adjustedTime

  return adjustedTime
end

function AntiAFK:getGamepadState()
  -- build table manually becuase g_inputBinding.gamepadInputState appears to be unreliable
  local state = {}

  for gamepadIndex = 0, g_inputBinding.numGamepads - 1 do
    for a = 0,13 do
      local deadzone = 0
      if g_inputBinding.devicesByInternalId[gamepadIndex].deadzones[a] ~= nil then
        deadzone = g_inputBinding.devicesByInternalId[gamepadIndex].deadzones[a]
      end
      local value = getInputAxis(a, gamepadIndex)

      if a < 10 then
        -- deadzone in middle
        table.insert(state, (value >= deadzone or value <= -deadzone) and value or 0)
      else
        -- deadzone at start of travel
        table.insert(state, value <= (1 - deadzone) and value or 1)
      end
    end
    for b = 0,31 do
      table.insert(state, getInputButton(b, gamepadIndex))
    end
  end

  return table.concat(state, ",")
end

function AntiAFK:resetText()
  setTextColor(1, 1, 1, 1)
  setTextAlignment(RenderText.ALIGN_LEFT)
  setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_BASELINE)
  setTextBold(false)
end

function AntiAFK:renderWarningText()
  local timeSinceInput = self:getTime() - self.lastInputTime
  local warningTime = self.afkTimeout - self.afkWarningTime

  if timeSinceInput > warningTime then
    local timeRemaining = self.lastInputTime + self.afkTimeout - self:getTime()

    if timeRemaining > 0 then
      local afkText = g_i18n:getText("AntiAFK_afkText")
      local secondsRemaining = math.ceil(timeRemaining / 1000)
      local countdownText = g_i18n:getText("AntiAFK_countdownText"):format(secondsRemaining, (secondsRemaining == 1 and g_i18n:getText("AntiAFK_countdownTextSecond") or g_i18n:getText("AntiAFK_countdownTextSeconds")))
      local green = math.abs(1 - ((timeRemaining / 250) % 2))

      setTextAlignment(RenderText.ALIGN_CENTER)
      setTextBold(true)

      setTextColor(1, green, 0, 1)
      setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_BOTTOM)
      renderText(0.5, 0.53, self.textSize, afkText)

      setTextColor(1, 1 - green, 0, 1)
      setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_TOP)
      renderText(0.5, 0.48, self.textSize, countdownText)

      self.resetText()
    end
  end
end

function AntiAFK:toggleDebugMode()
  AntiAFK.debugMode = not AntiAFK.debugMode
end

function AntiAFK:loadMap()
  FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, self.registerActionEvents)
  g_mpLoadingScreen.onReadyToStart = Utils.appendedFunction(g_mpLoadingScreen.onReadyToStart, self.onReadyToStart)
  self.enabled = g_dedicatedServer == nil and g_currentMission.missionDynamicInfo.isMultiplayer and g_currentMission.missionDynamicInfo.isClient
end

function AntiAFK:draw()
  if self.enabled then
    local timeRemaining = self.lastInputTime + self.afkTimeout - self:getTime()

    if not self.debugMode and not g_noHudModeEnabled then
      setTextColor(1, 1, 1, 0.5)
      setTextBold(true)
      setTextAlignment(RenderText.ALIGN_RIGHT)
      setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_BOTTOM)
      renderText(0.995, 0.005, 0.01, g_i18n:getText("AntiAFK_activeMessageText"))

      self.resetText()
    end

    local timeSinceInput = self:getTime() - self.lastInputTime
    local warningTime = self.afkTimeout - self.afkWarningTime

    if timeSinceInput > warningTime then
      setOverlayColor(self.overlay.overlayId, 0, 0, 0, 1 - (timeRemaining / self.afkWarningTime))
      self.overlay:setPosition(0, 0)
      self.overlay:setDimension(1, 1)
      self.overlay:render()

      self:renderWarningText()
    end
  end

  if self.debugMode then
    local timeRemaining = self.lastInputTime + self.afkTimeout - self:getTime()
    local secondsRemaining = math.ceil(timeRemaining / 1000)

    setTextColor(1, 0, 0, 1)
    setTextBold(true)
    setTextAlignment(RenderText.ALIGN_RIGHT)
    setTextVerticalAlignment(RenderText.VERTICAL_ALIGN_BOTTOM)
    renderText(0.995, 0.005, 0.01, g_i18n:getText("AntiAFK_debugMessageText"):format(
      (self.enabled and g_i18n:getText("AntiAFK_debugMessageEnabled") or g_i18n:getText("AntiAFK_debugMessageDisabled")),
      (self.enabled and self.lastInputType ~= "") and " <" .. self.lastInputType .. ">" or "",
      self.enabled and " (" .. tostring(secondsRemaining) .. ")" or "")
    )

    self.resetText()
  end
end

function AntiAFK:update(dt)
  if self.enabled then
    local gamepadState = AntiAFK.getGamepadState()
    if gamepadState ~= self.lastGamepadState then
      self.lastInputTime = self:getTime()
      self.lastInputType = g_i18n:getText("AntiAFK_debugMessageInpuyTypeGamepad")
    end

    self.lastGamepadState = gamepadState

    local timeSinceInput = self:getTime() - self.lastInputTime
    local warningTime = self.afkTimeout - self.afkWarningTime

    if timeSinceInput > warningTime then
      self:renderWarningText()

      if self:getTime() - self.lastInputTime > self.afkTimeout then
        self.enabled = false
        g_client:getServerConnection():sendEvent(ChatEvent.new(g_i18n:getText("AntiAFK_removeMessageText"), g_currentMission.playerNickname, FarmManager.SPECTATOR_FARM_ID, g_currentMission.playerUserId))
        -- leave game on next tick to prevent errors
        FSBaseMission.update = Utils.appendedFunction(FSBaseMission.update, AntiAFK.leaveGame)
      end
    end
  end
end

function AntiAFK:mouseEvent(posX, posY, isDown, isUp, button)
  if self.enabled then
    self.lastInputTime = self:getTime()
    self.lastInputType = g_i18n:getText("AntiAFK_debugMessageInpuyTypeMouse")
  end
end

function AntiAFK:keyEvent(unicode, sym, modifier, isDown)
  if self.enabled then
    self.lastInputTime = self:getTime()
    self.lastInputType = g_i18n:getText("AntiAFK_debugMessageInpuyTypeKeyboard")
  end
end

function AntiAFK:registerActionEvents()
  if g_dedicatedServer == nil then
    local _, actionEventId = g_inputBinding:registerActionEvent(InputAction.TOGGLE_ANTIAFK_DEBUG, AntiAFK, AntiAFK.toggleDebugMode, false, true, false, true)
    g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW)
    g_inputBinding:setActionEventTextVisibility(actionEventId, false)
  end
end

function AntiAFK:onReadyToStart()
  if AntiAFK.enabled then
    AntiAFK.lastInputTime = AntiAFK:getTime()
    self:onClickOk()
  end
end

function AntiAFK:leaveGame()
  if not AntiAFK.requestLeave then
    AntiAFK.requestLeave = true
    g_currentMission.inGameMenu:leaveCurrentGame()
  end
end

addModEventListener(AntiAFK)